@@ -1,12 +1,13 @@
 # Automated tests for the `executor' module.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: November 13, 2015
+# Last Change: January 14, 2016
 # URL: https://executor.readthedocs.org
 
 """Automated tests for the `executor` package."""
 
 # Standard library modules.
+import datetime
 import logging
 import os
 import random
@@ -33,24 +34,22 @@ from executor import (
     quote,
     which,
 )
-from executor.cli import (
-    CommandTimedOut,
-    apply_fudge_factor,
-    get_lock_path,
-    main,
-    run_command,
-)
+from executor.cli import main
 from executor.concurrent import CommandPool, CommandPoolFailed
 from executor.contexts import LocalContext, RemoteContext
 from executor.ssh.client import (
     DEFAULT_CONNECT_TIMEOUT,
     RemoteCommand,
     RemoteCommandFailed,
+    RemoteCommandNotFound,
     RemoteConnectFailed,
     foreach,
+    remote,
 )
 from executor.ssh.server import SSHServer
 
+MISSING_COMMAND = 'a-program-name-that-no-one-would-ever-use'
+
 
 class ExecutorTestCase(unittest.TestCase):
 
@@ -87,7 +86,7 @@ class ExecutorTestCase(unittest.TestCase):
     def test_program_searching(self):
         """Make sure which() works as expected."""
         assert which('python')
-        assert not which('a-program-name-that-no-one-would-ever-use')
+        assert not which(MISSING_COMMAND)
 
     def test_status_code_checking(self):
         """Make sure that status code handling is sane."""
@@ -101,9 +100,9 @@ class ExecutorTestCase(unittest.TestCase):
         self.assertEqual(e.command.command_line, ['bash', '-c', 'exit 42'])
         self.assertEqual(e.returncode, 42)
         # Make sure CommandNotFound exceptions work for shell commands.
-        self.assertRaises(CommandNotFound, execute, 'a-program-name-that-no-one-would-ever-use')
+        self.assertRaises(CommandNotFound, execute, MISSING_COMMAND)
         # Make sure CommandNotFound exceptions work for non-shell commands.
-        self.assertRaises(CommandNotFound, execute, 'a-program-name-that-no-one-would-ever-use', 'just-an-argument')
+        self.assertRaises(CommandNotFound, execute, MISSING_COMMAND, 'just-an-argument')
 
     def test_stdin(self):
         """Make sure standard input can be provided to external commands."""
@@ -338,6 +337,15 @@ class ExecutorTestCase(unittest.TestCase):
         cmd.wait()
         assert cmd.output == random_value
 
+    def test_callback_evaluation(self):
+        """Make sure result processing callbacks work as expected."""
+        result = execute('echo', str(time.time()), callback=self.coerce_timestamp)
+        assert isinstance(result, datetime.datetime)
+
+    def coerce_timestamp(self, cmd):
+        """Callback for :func:`test_callback_evaluation()`."""
+        return datetime.datetime.fromtimestamp(float(cmd.output))
+
     def test_suspend_and_resume_signals(self):
         """Test the sending of ``SIGSTOP``, ``SIGCONT`` and ``SIGTERM`` signals."""
         # Spawn a child that will live for a minute.
@@ -563,21 +571,28 @@ class ExecutorTestCase(unittest.TestCase):
         # Make sure invalid SSH aliases raise the expected type of exception.
         self.assertRaises(
             RemoteConnectFailed,
-            lambda: RemoteCommand('this.domain.surely.wont.exist.right', 'date', silent=True).start(),
+            remote, 'this.domain.surely.wont.exist.right', 'date', silent=True,
         )
 
+    def test_remote_command_missing(self):
+        """Make sure a specific exception is raised when a remote command is missing."""
+        with SSHServer() as server:
+            self.assertRaises(
+                RemoteCommandNotFound,
+                remote, '127.0.0.1', MISSING_COMMAND,
+                **server.client_options
+            )
+
     def test_remote_working_directory(self):
         """Make sure remote working directories can be set."""
         with SSHServer() as server:
             some_random_directory = tempfile.mkdtemp()
             try:
-                cmd = RemoteCommand('127.0.0.1',
-                                    'pwd',
-                                    capture=True,
-                                    directory=some_random_directory,
-                                    **server.client_options)
-                cmd.start()
-                assert cmd.output == some_random_directory
+                output = remote('127.0.0.1', 'pwd',
+                                capture=True,
+                                directory=some_random_directory,
+                                **server.client_options)
+                assert output == some_random_directory
             finally:
                 shutil.rmtree(some_random_directory)
 
